package commands;

import db.DbHandler;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Scanner;

/**
 * Created by luigibellomarini on 20/01/16.
 */
public class SpedizioneCommand implements Command {

    private String getIdCategoriaByNome(String nome) throws CommandRuntimeException {
        PreparedStatement categoriaSt = null;
        ResultSet rs = null;

        try {
            categoriaSt = DbHandler.getInstance()
                    .getPreparedStatement("select \"Codice\" " +
                            "from \"RomaTreEx\".\"CategoriaSpedizione\" " +
                            "where " +
                            "\"Descrizione\" = ?");
            categoriaSt.setString(1,nome);
            rs = categoriaSt.executeQuery();

            if (rs.next()) {
                return rs.getString(1);
            } else
                throw new CommandRuntimeException("Categoria non trovata");

        } catch (SQLException e) {
            throw new CommandRuntimeException("Categoria non trovata");
        } finally {
            try {
                rs.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }

            try {
                categoriaSt.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }
    }

    private String getStatoSpedizioneByNome(String nome) throws CommandRuntimeException {
        PreparedStatement statoSt = null;
        ResultSet rs = null;

        try {
            statoSt = DbHandler.getInstance()
                    .getPreparedStatement("select \"Codice\" " +
                            "from \"RomaTreEx\".\"StatoSpedizione\" " +
                            "where " +
                            "\"Descrizione\" = ?");
            statoSt.setString(1,nome);
            rs = statoSt.executeQuery();

            if (rs.next()) {
                return rs.getString(1);
            } else
                throw new CommandRuntimeException("Stato non trovato");

        } catch (SQLException e) {
            throw new CommandRuntimeException("Stato non trovato");
        } finally {
            try {
                rs.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }
            try {
                statoSt.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }
    }

    private String getCoordinateLuogoByNome(String nome) throws CommandRuntimeException {
        PreparedStatement luogoSt = null;
        ResultSet rs = null;

        try {
            luogoSt = DbHandler.getInstance()
                    .getPreparedStatement("select \"Coordinate\" " +
                            "from \"RomaTreEx\".\"Luogo\" " +
                            "where " +
                            "\"Descrizione\" = ?");
            luogoSt.setString(1,nome);
            rs = luogoSt.executeQuery();
            if (rs.next()) {
                return rs.getString(1);
            } else
                throw new CommandRuntimeException("Luogo non trovato");

        } catch (SQLException e) {
            throw new CommandRuntimeException("Luogo non trovato");
        } finally {
            try {
                rs.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }
            try {
                luogoSt.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }
    }

    private int getIdSoggettoByNome(String nome) throws CommandRuntimeException {
        PreparedStatement soggettoSt = null;
        ResultSet rs = null;

        try {
                soggettoSt = DbHandler.getInstance()
                        .getPreparedStatement("select \"Codice\" " +
                                "from \"RomaTreEx\".\"Soggetto\" " +
                                "where \"Nome\" = ?");

                soggettoSt.setString(1,nome);
                rs = soggettoSt.executeQuery();
                if(rs.next()) {
                    return rs.getInt(1);
                } else
                    throw new CommandRuntimeException("Soggetto non trovato");

        } catch (SQLException e) {
            throw new CommandRuntimeException("Soggetto non trovato");
        } finally {
            try {
                rs.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }

            try {
                soggettoSt.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }

        }

    }

    private String getIndirizzoBySoggetto(int codiceSoggetto) throws CommandRuntimeException {

        PreparedStatement indirizzoSt = null;
        ResultSet rs = null;

        try {
            indirizzoSt = DbHandler.getInstance()
                    .getPreparedStatement("select \"Indirizzo\" " +
                            "from \"RomaTreEx\".\"Soggetto\" " +
                            "where " +
                            "\"Codice\" = ?");
            indirizzoSt.setInt(1,codiceSoggetto);
            rs = indirizzoSt.executeQuery();
            if (rs.next()) {
                return rs.getString(1);
            } else
                throw new CommandRuntimeException("Soggetto non troato");

        } catch (SQLException e) {
            throw new CommandRuntimeException("Soggetto non trovato");
        } finally {
            try {
                indirizzoSt.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }

        }
    }



    @Override
    public void executeCommand() throws CommandRuntimeException {
        Scanner sc = new Scanner(System.in);


        System.out.print("indirizzo destinazione> ");
        String indirizzoDestinazione = sc.nextLine();

        System.out.print("nome categoria (overnight, celere, espressa, ordinaria)> ");
        String categoria = sc.nextLine(); // to resolve

        System.out.print("nome mittente> ");
        String mittente = sc.nextLine();

        System.out.print("nome destinatario> ");
        String destinatario = sc.nextLine();

        System.out.print("a domicilio (1: SI, 0: NO)> ");
        int aDomicilio = Integer.parseInt(sc.nextLine());
        String indirizzoRaccolta = null;

        if (aDomicilio==1) {
            indirizzoRaccolta = this.getIndirizzoBySoggetto(this.getIdSoggettoByNome(mittente));

        } else {
            indirizzoRaccolta = "via del Magazzino, 1 00100 Roma";
        }


        PreparedStatement spedizioneSt = null;

        try {


            spedizioneSt = DbHandler.getInstance().getPreparedStatement("INSERT INTO \"RomaTreEx\".\"Spedizione\"(" +
                    "\"Codice\"," +
                    "\"ADomicilio\"," +
                    "\"IndirizzoDestinazione\"," +
                    "\"IndirizzoRaccolta\"," +
                    "\"Categoria\"," +
                    "\"Mittente\","+
                    "\"Destinatario\"," +
                    "\"Stato\") VALUES (nextval('\"RomaTreEx\".\"SEQ_Spedizione\"'),?,?,?,?,?,?,?)");

            spedizioneSt.setInt(1, aDomicilio);
            spedizioneSt.setString(2, indirizzoDestinazione);
            spedizioneSt.setString(3, indirizzoRaccolta);
            spedizioneSt.setString(4, this.getIdCategoriaByNome(categoria));
            spedizioneSt.setInt(5, this.getIdSoggettoByNome(mittente));
            spedizioneSt.setInt(6, this.getIdSoggettoByNome(destinatario));
            spedizioneSt.setString(7, this.getStatoSpedizioneByNome("non spedita"));

            spedizioneSt.executeUpdate();

            PreparedStatement posizioneSt = DbHandler.getInstance().getPreparedStatement("INSERT INTO \"RomaTreEx\".\"Posizione\" (" +
                    "\"Spedizione\"," +
                    "\"Luogo\"," +
                    "\"DataOra\") VALUES (currval('\"RomaTreEx\".\"SEQ_Spedizione\"'),?,current_timestamp)");

            posizioneSt.setString(1, this.getCoordinateLuogoByNome(indirizzoRaccolta));
            posizioneSt.executeUpdate();

            DbHandler.getInstance().commitTransaction();


        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            try {
                spedizioneSt.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }
            try {
                DbHandler.getInstance().closeConnection();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }


    }


}
